// ******************************************************* //
// G L O B A L   V A R I A B L E S                         //
// ******************************************************* //

matrix g_ViewProjectionInverseMatrix;	// View transform
matrix g_previousViewProjectionMatrix;	// Projection transform

extern int GNumSamples;				// Number of samples to take
extern float GDenom;					// Velocity dampning

extern float SX;						// Screen mode x direction
extern float SY;						// Screen mode y direction

// ******************************************************* //
// T E X T U R E   S A M P L E R S                         //
// ******************************************************* //

texture scene;
sampler2D SceneSampler = sampler_state
{
	Texture = <scene>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
};

texture depthTexture;
sampler2D DepthSampler = sampler_state
{
	Texture = <depthTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
};

// ******************************************************* //
// D A T A   S T R U C T U R E S                           //
// ******************************************************* //

struct VS_OUT
{
	float4 pos : SV_POSITION;
	float2 tx : TEXCOORD0;
};

// ******************************************************* //
// V E R T E X   S H A D E R S                             //
// ******************************************************* //

VS_OUT VS_BASIC(float4 Pos : POSITION, float2 UV : TEXCOORD0)
{
	// FS Quad already in screen space (no transform rqd)

	VS_OUT vOut;
	vOut.pos = Pos;
	vOut.tx = UV;
	return vOut;
}

// ******************************************************* //
// P I X E L   S H A D E R S                               //
// ******************************************************* //

float4 PS_WALK(VS_OUT pIn) : SV_TARGET
{
	float2 texCoord = pIn.tx;

	// Get the depth value for this fragment
	float zOverW = tex2D(DepthSampler, texCoord);

	// V is the viewports position at this fragent (-1 to 1)
	float4 V = float4(texCoord.x * 2 - 1, (1 - texCoord.y) * 2 - 1, zOverW, 1);

	// Transform by the view-projection inverse
	float4 D = mul(V, g_ViewProjectionInverseMatrix);

	// Divide by w to get the world position.
	float4 worldPos = D / D.w;

	/////////////////

	// Current viewport position
	float4 currentPos = V;

	// Use the world position, and transform by the previous view-projection matrix
	float4 previousPos = mul(worldPos, g_previousViewProjectionMatrix);

	// Convert to nonhomogeneous points [-1,1] by dividing by w
	previousPos /= previousPos.w;

	// Use this frame's position and last frame's to compute the pixel velocity
	float2 velocity = (currentPos - previousPos)/GDenom;

	///////////////////////

	// Get the initial color at this pixel
	float4 color = tex2D(SceneSampler, texCoord);
	texCoord += velocity;

	for(int i = 1; i < 10; ++i, texCoord += velocity)
	{
		if(i>GNumSamples)
			break;

		// Sample the color buffer along the velocity vector
		float4 currentColor = tex2D(SceneSampler, texCoord);

		// Add the current color to our color sum
		color += currentColor;
	}

	// Average out the final blur colour
	float4 finalColor = color / GNumSamples;
	return finalColor;
}

float4 PS_SCREEN(VS_OUT pIn) : SV_TARGET
{
	float2 texCoord = pIn.tx;

	// Use arbitrary screen space direction
	float2 velocity = float2(SX, SY);

	// Get the initial color at this pixel
	float4 color = tex2D(SceneSampler, texCoord);
	texCoord += velocity;

	for(int i = 1; i < 10; ++i, texCoord += velocity)
	{
		if(i>GNumSamples)
			break;

		// Sample the color buffer along the velocity vector
		float4 currentColor = tex2D(SceneSampler, texCoord);

		// Add the current color to our color sum
		color += currentColor;
	}

	// Average out the final blur colour
	float4 finalColor = color / GNumSamples;
	return finalColor;
}

// ******************************************************* //
// T E C H N I Q U E S                                     //
// ******************************************************* //

// Technique Definition
technique PPMotionBlurWalkMode
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_BASIC();
		PixelShader = compile ps_3_0 PS_WALK();
	}
}

// Technique Definition
technique PPMotionBlurDriveMode
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_BASIC();
		PixelShader = compile ps_3_0 PS_WALK();
	}
}

// Technique Definition
technique PPMotionBlurScreenMode
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_BASIC();
		PixelShader = compile ps_3_0 PS_SCREEN();
	}
}