// ******************************************************* //
// G L O B A L   V A R I A B L E S                         //
// ******************************************************* //

// Object related
matrix  GWorldMTX;			// World transform

// Camera Related
float3  GCamPos;			// Camera Position
matrix  GViewMTX;			// View transform
matrix  GProjectionMTX;		// Projection transform

// Light Related
float3  LightPos;			// Light position
float3  LightDir;			// Light direction
float4  LightAmbient;		// Ambient parameter
float4  LightDiffuse;		// Diffuse parameter
float4  LightSpec;			// Specular parameter (0=r,1=g,2=b,3=exponent)

// Material Related
float4  MaterialAmbient;	// Diffuse  light reflected
float4  MaterialDiffuse;	// Diffuse  light reflected
float4  MaterialSpec;		// Specular light reflected
float   MaterialSpecPow;	// Specular parameter (shinyness)
texture MaterialTexture;	// Material texture

// Misc
float   ElapsedTime;		// Game time
float   RandomNumber;		// Random number (for car bounce)

// ******************************************************* //
// T E X T U R E   S A M P L E R S                         //
// ******************************************************* //

sampler2D ColorSamp = sampler_state
{
	Texture = <MaterialTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

// ******************************************************* //
// D A T A   S T R U C T U R E S                           //
// ******************************************************* //

struct VS_OUT
{
	float4 pos : SV_POSITION;
	float3 posW : POSITION1;				//Vertex position (world space)
	float3 normalW : NORMAL;				//Vertex notmal (world space)
	float2 tx : TEXCOORD0;
};

// ******************************************************* //
// V E R T E X   S H A D E R S                             //
// ******************************************************* //

VS_OUT VS_BASIC(float4 Pos : POSITION, float4 Norm : NORMAL, float2 UV : TEXCOORD0)
{
	VS_OUT vOut;	//To hold result

	//Transform vertex and normal to world space
	vOut.posW = mul(float4(Pos.x, Pos.y, Pos.z, 0), GWorldMTX);
	vOut.normalW = mul(float4(Norm.x, Norm.y, Norm.z, 0.0f), GWorldMTX);

	vOut.pos = mul( Pos, GWorldMTX );
	vOut.pos = mul( vOut.pos, GViewMTX );
	vOut.pos = mul( vOut.pos, GProjectionMTX );

	vOut.tx = UV;
	return vOut;
}

VS_OUT VS_CAR_BODY(float4 Pos : POSITION, float4 Norm : NORMAL, float2 UV : TEXCOORD0)
{
	VS_OUT vOut = VS_BASIC(Pos, Norm, UV);	//To hold result

	// Make body of car bounce (to simulate uneven road surface)
	vOut.pos.y += sin(ElapsedTime*4 + cos(ElapsedTime/2) + RandomNumber/20)/15;

	return vOut;
}

// ******************************************************* //
// P I X E L   S H A D E R S                               //
// ******************************************************* //

float4 PS_PHONG(VS_OUT pIn) : SV_TARGET
{
	pIn.normalW = normalize(pIn.normalW);
	float4 baseColour =  tex2D(ColorSamp, pIn.tx);
	if(baseColour.x+baseColour.y+baseColour.z==0)
		baseColour = MaterialDiffuse;

	float3 toEye = normalize(GCamPos - pIn.posW);

	float4 outColour = float4(0,0,0,1);

	// Ambient
	outColour = baseColour * LightAmbient;

	float diffuseFactor = dot( -LightDir, pIn.normalW);

	if(diffuseFactor > 0)
	{
		// Diffuse
		outColour += diffuseFactor * MaterialDiffuse * LightDiffuse;

		// Spec
		float3 R = reflect(LightDir, pIn.normalW);
		float specFactor = pow(max(dot(R, toEye), 0.0f), MaterialSpecPow);
		outColour += MaterialSpec * specFactor * LightSpec;
	}
	return outColour;
}

// ******************************************************* //
// T E C H N I Q U E S                                     //
// ******************************************************* //

technique Render
{
	pass P0
	{
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
		VertexShader = compile vs_3_0 VS_BASIC();
		PixelShader = compile ps_3_0 PS_PHONG();
	}
}

technique RenderBody
{
	pass P0
	{
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
		VertexShader = compile vs_3_0 VS_CAR_BODY();
		PixelShader = compile ps_3_0 PS_PHONG();
	}
}